@@ -1,13 +1,14 @@
 extern crate oci_distribution;
 
-use crate::util::{cached_file, format_output, Output, OutputKind};
+use crate::util::{cached_file, CommandOutput, OutputKind};
+use anyhow::{anyhow, bail, Result};
+use clap::{Parser, Subcommand};
 use log::{debug, info, warn};
 use oci_distribution::{client::*, secrets::RegistryAuth, Reference};
 use provider_archive::ProviderArchive;
 use serde_json::json;
 use spinners::{Spinner, Spinners};
-use std::{fs::File, io::prelude::*};
-use structopt::{clap::AppSettings, StructOpt};
+use std::{collections::HashMap, fs::File, io::prelude::*};
 
 const PROVIDER_ARCHIVE_MEDIA_TYPE: &str = "application/vnd.wasmcloud.provider.archive.layer.v1+par";
 const PROVIDER_ARCHIVE_CONFIG_MEDIA_TYPE: &str =
@@ -25,99 +26,78 @@ pub(crate) enum SupportedArtifacts {
     Wasm,
 }
 
-#[derive(Debug, StructOpt, Clone)]
-#[structopt(
-    global_settings(&[AppSettings::ColoredHelp, AppSettings::VersionlessSubcommands]),
-    name = "reg")]
-pub(crate) struct RegCli {
-    #[structopt(flatten)]
-    command: RegCliCommand,
-}
-
-impl RegCli {
-    pub(crate) fn command(self) -> RegCliCommand {
-        self.command
-    }
-}
-
-#[derive(Debug, Clone, StructOpt)]
+#[derive(Debug, Clone, Subcommand)]
 pub(crate) enum RegCliCommand {
     /// Pull an artifact from an OCI compliant registry
-    #[structopt(name = "pull")]
+    #[clap(name = "pull")]
     Pull(PullCommand),
     /// Push an artifact to an OCI compliant registry
-    #[structopt(name = "push")]
+    #[clap(name = "push")]
     Push(PushCommand),
     /// Ping (test url) to see if the OCI url has an artifact
-    #[structopt(name = "ping")]
+    #[clap(name = "ping")]
     Ping(PingCommand),
 }
 
-#[derive(StructOpt, Debug, Clone)]
+#[derive(Parser, Debug, Clone)]
 pub(crate) struct PullCommand {
     /// URL of artifact
-    #[structopt(name = "url")]
+    #[clap(name = "url")]
     pub(crate) url: String,
 
     /// File destination of artifact
-    #[structopt(long = "destination")]
+    #[clap(long = "destination")]
     pub(crate) destination: Option<String>,
 
     /// Digest to verify artifact against
-    #[structopt(short = "d", long = "digest")]
+    #[clap(short = 'd', long = "digest")]
     pub(crate) digest: Option<String>,
 
     /// Allow latest artifact tags
-    #[structopt(long = "allow-latest")]
+    #[clap(long = "allow-latest")]
     pub(crate) allow_latest: bool,
 
-    #[structopt(flatten)]
-    pub(crate) output: Output,
-
-    #[structopt(flatten)]
+    #[clap(flatten)]
     pub(crate) opts: AuthOpts,
 }
 
-#[derive(StructOpt, Debug, Clone)]
+#[derive(Parser, Debug, Clone)]
 pub(crate) struct PushCommand {
     /// URL to push artifact to
-    #[structopt(name = "url")]
+    #[clap(name = "url")]
     pub(crate) url: String,
 
     /// Path to artifact to push
-    #[structopt(name = "artifact")]
+    #[clap(name = "artifact")]
     pub(crate) artifact: String,
 
     /// Path to config file, if omitted will default to a blank configuration
-    #[structopt(short = "c", long = "config")]
+    #[clap(short = 'c', long = "config")]
     pub(crate) config: Option<String>,
 
     /// Allow latest artifact tags
-    #[structopt(long = "allow-latest")]
+    #[clap(long = "allow-latest")]
     pub(crate) allow_latest: bool,
 
-    #[structopt(flatten)]
-    pub(crate) output: Output,
-
-    #[structopt(flatten)]
+    #[clap(flatten)]
     pub(crate) opts: AuthOpts,
 }
 
-#[derive(StructOpt, Debug, Clone)]
+#[derive(Parser, Debug, Clone)]
 pub(crate) struct PingCommand {
     /// URL of artifact
-    #[structopt(name = "url")]
+    #[clap(name = "url")]
     pub(crate) url: String,
 
-    #[structopt(flatten)]
+    #[clap(flatten)]
     pub(crate) opts: AuthOpts,
 }
 
-#[derive(StructOpt, Debug, Clone)]
+#[derive(Parser, Debug, Clone)]
 pub(crate) struct AuthOpts {
     /// OCI username, if omitted anonymous authentication will be used
-    #[structopt(
-        short = "u",
+    #[clap(
+        short = 'u',
         long = "user",
         env = "WASH_REG_USER",
         hide_env_values = true
@@ -125,8 +105,8 @@ pub(crate) struct AuthOpts {
     pub(crate) user: Option<String>,
 
     /// OCI password, if omitted anonymous authentication will be used
-    #[structopt(
-        short = "p",
+    #[clap(
+        short = 'p',
         long = "password",
         env = "WASH_REG_PASSWORD",
         hide_env_values = true
@@ -134,23 +114,27 @@ pub(crate) struct AuthOpts {
     pub(crate) password: Option<String>,
 
     /// Allow insecure (HTTP) registry connections
-    #[structopt(long = "insecure")]
+    #[clap(long = "insecure")]
     pub(crate) insecure: bool,
 }
 
 pub(crate) async fn handle_command(
     command: RegCliCommand,
-) -> Result<String, Box<dyn ::std::error::Error>> {
+    output_kind: OutputKind,
+) -> Result<CommandOutput> {
     match command {
-        RegCliCommand::Pull(cmd) => handle_pull(cmd).await,
-        RegCliCommand::Push(cmd) => handle_push(cmd).await,
+        RegCliCommand::Pull(cmd) => handle_pull(cmd, output_kind).await,
+        RegCliCommand::Push(cmd) => handle_push(cmd, output_kind).await,
         RegCliCommand::Ping(cmd) => handle_ping(cmd).await,
     }
 }
 
-pub(crate) async fn handle_pull(cmd: PullCommand) -> Result<String, Box<dyn ::std::error::Error>> {
-    let image: Reference = cmd.url.parse().unwrap();
-    let spinner = match cmd.output.kind {
+pub(crate) async fn handle_pull(
+    cmd: PullCommand,
+    output_kind: OutputKind,
+) -> Result<CommandOutput> {
+    let image: Reference = cmd.url.parse()?;
+    let spinner = match output_kind {
         OutputKind::Text => Some(Spinner::new(
             &Spinners::Dots12,
             format!(" Downloading {} ...", image.whole()),
@@ -173,14 +157,14 @@ pub(crate) async fn handle_pull(cmd: PullCommand) -> Result<String, Box<dyn ::st
     if spinner.is_some() {
         spinner.unwrap().stop();
     }
-
-    Ok(format_output(
+    let mut map = HashMap::new();
+    map.insert("file".to_string(), json!(outfile));
+    Ok(CommandOutput::new(
         format!(
             "\n{} Successfully pulled and validated {}",
             SHOWER_EMOJI, outfile
         ),
-        json!({"result": "success", "file": outfile}),
-        &cmd.output.kind,
+        map,
     ))
 }
 
@@ -194,7 +178,7 @@ pub(crate) async fn get_artifact(
     password: Option<String>,
     insecure: bool,
     no_cache: bool,
-) -> Result<Vec<u8>, Box<dyn ::std::error::Error>> {
+) -> Result<Vec<u8>> {
     if let Ok(mut local_artifact) = File::open(url.clone()) {
         let mut buf = Vec::new();
         local_artifact.read_to_end(&mut buf)?;
@@ -215,13 +199,12 @@ pub(crate) async fn pull_artifact(
     user: Option<String>,
     password: Option<String>,
     insecure: bool,
-) -> Result<Vec<u8>, Box<dyn ::std::error::Error>> {
+) -> Result<Vec<u8>> {
     let image: Reference = url.parse()?;
 
     if image.tag().unwrap_or("latest") == "latest" && !allow_latest {
-        return Err(
+        bail!(
             "Pulling artifacts with tag 'latest' is prohibited. This can be overriden with a flag"
-                .into(),
         );
     };
 
@@ -255,9 +238,9 @@ pub(crate) async fn pull_artifact(
     };
 
     match (digest, image_data.digest) {
-        (Some(digest), Some(image_digest)) if digest != image_digest => {
-            Err("Image digest did not match provided digest, aborting")
-        }
+        (Some(digest), Some(image_digest)) if digest != image_digest => Err(anyhow!(
+            "Image digest did not match provided digest, aborting"
+        )),
         _ => {
             debug!("Image digest validated against provided digest");
             Ok(())
@@ -267,13 +250,12 @@ pub(crate) async fn pull_artifact(
     Ok(image_data
         .layers
         .iter()
-        .map(|l| l.data.clone())
-        .flatten()
+        .flat_map(|l| l.data.clone())
         .collect::<Vec<_>>())
 }
 
-pub(crate) async fn handle_ping(cmd: PingCommand) -> Result<String, Box<dyn ::std::error::Error>> {
-    let image: Reference = cmd.url.parse().unwrap();
+pub(crate) async fn handle_ping(cmd: PingCommand) -> Result<CommandOutput> {
+    let image: Reference = cmd.url.parse()?;
     let mut client = Client::new(ClientConfig {
         protocol: if cmd.opts.insecure {
             ClientProtocol::Http
@@ -287,14 +269,14 @@ pub(crate) async fn handle_ping(cmd: PingCommand) -> Result<String, Box<dyn ::st
         _ => RegistryAuth::Anonymous,
     };
     let (_, _) = client.pull_manifest(&image, &auth).await?;
-    Ok("Ok".to_string())
+    Ok(CommandOutput::from("Pong!"))
 }
 
 pub(crate) fn write_artifact(
     artifact: &[u8],
     image: &Reference,
     output: Option<String>,
-) -> Result<String, Box<dyn ::std::error::Error>> {
+) -> Result<String> {
     let file_extension = match validate_artifact(artifact, image.repository())? {
         SupportedArtifacts::Par => PROVIDER_ARCHIVE_FILE_EXTENSION,
         SupportedArtifacts::Wasm => WASM_FILE_EXTENSION,
@@ -318,50 +300,44 @@ pub(crate) fn write_artifact(
 
 /// Helper function to determine artifact type and validate that it is
 /// a valid artifact of that type
-pub(crate) fn validate_artifact(
-    artifact: &[u8],
-    name: &str,
-) -> Result<SupportedArtifacts, Box<dyn ::std::error::Error>> {
+pub(crate) fn validate_artifact(artifact: &[u8], name: &str) -> Result<SupportedArtifacts> {
     match validate_actor_module(artifact, name) {
         Ok(_) => Ok(SupportedArtifacts::Wasm),
         Err(_) => match validate_provider_archive(artifact, name) {
             Ok(_) => Ok(SupportedArtifacts::Par),
-            Err(_) => Err("Unsupported artifact type".into()),
+            Err(_) => bail!("Unsupported artifact type"),
         },
     }
 }
 
 /// Attempts to inspect the claims of an actor module
 /// Will fail without actor claims, or if the artifact is invalid
-fn validate_actor_module(
-    artifact: &[u8],
-    module: &str,
-) -> Result<(), Box<dyn ::std::error::Error>> {
+fn validate_actor_module(artifact: &[u8], module: &str) -> Result<()> {
     match wascap::wasm::extract_claims(&artifact) {
         Ok(Some(_token)) => Ok(()),
-        Ok(None) => Err(format!("No capabilities discovered in actor module : {}", &module).into()),
-        Err(e) => Err(Box::new(e)),
+        Ok(None) => bail!("No capabilities discovered in actor module : {}", &module),
+        Err(e) => Err(anyhow!("{}", e)),
     }
 }
 
 /// Attempts to unpack a provider archive
 /// Will fail without claims or if the archive is invalid
-fn validate_provider_archive(
-    artifact: &[u8],
-    archive: &str,
-) -> Result<(), Box<dyn ::std::error::Error>> {
+fn validate_provider_archive(artifact: &[u8], archive: &str) -> Result<()> {
     match ProviderArchive::try_load(artifact) {
         Ok(_par) => Ok(()),
-        Err(_e) => Err(format!("Invalid provider archive : {}", archive).into()),
+        Err(_e) => bail!("Invalid provider archive : {}", archive),
     }
 }
 
-pub(crate) async fn handle_push(cmd: PushCommand) -> Result<String, Box<dyn ::std::error::Error>> {
+pub(crate) async fn handle_push(
+    cmd: PushCommand,
+    output_kind: OutputKind,
+) -> Result<CommandOutput> {
     if cmd.url.starts_with("localhost:") && !cmd.opts.insecure {
         warn!(" Unless an SSL certificate has been installed, pushing to localhost without the --insecure option will fail")
     }
 
-    let spinner = match cmd.output.kind {
+    let spinner = match output_kind {
         OutputKind::Text => Some(Spinner::new(
             &Spinners::Dots12,
             format!(" Pushing {} to {} ...", cmd.artifact, cmd.url),
@@ -384,13 +360,15 @@ pub(crate) async fn handle_push(cmd: PushCommand) -> Result<String, Box<dyn ::st
     if spinner.is_some() {
         spinner.unwrap().stop();
     }
-    Ok(format_output(
+
+    let mut map = HashMap::new();
+    map.insert("url".to_string(), json!(cmd.url));
+    Ok(CommandOutput::new(
         format!(
-            "\n{} Successfully validated and pushed to {}",
+            "{} Successfully validated and pushed to {}",
             SHOWER_EMOJI, cmd.url
         ),
-        json!({"result": "success", "url": cmd.url}),
-        &cmd.output.kind,
+        map,
     ))
 }
 
@@ -402,13 +380,12 @@ pub(crate) async fn push_artifact(
     user: Option<String>,
     password: Option<String>,
     insecure: bool,
-) -> Result<(), Box<dyn ::std::error::Error>> {
-    let image: Reference = url.parse().unwrap();
+) -> Result<()> {
+    let image: Reference = url.parse()?;
 
     if image.tag().unwrap() == "latest" && !allow_latest {
-        return Err(
+        bail!(
             "Pushing artifacts with tag 'latest' is prohibited. This can be overriden with a flag"
-                .into(),
         );
     };
 
@@ -474,13 +451,18 @@ pub(crate) async fn push_artifact(
 
 #[cfg(test)]
 mod tests {
-    use super::{PullCommand, PushCommand, RegCli, RegCliCommand};
-    use crate::util::OutputKind;
-    use structopt::StructOpt;
+    use super::{PullCommand, PushCommand, RegCliCommand};
+    use clap::Parser;
 
     const ECHO_WASM: &str = "wasmcloud.azurecr.io/echo:0.2.0";
     const LOCAL_REGISTRY: &str = "localhost:5000";
 
+    #[derive(Debug, Parser)]
+    struct Cmd {
+        #[clap(subcommand)]
+        reg: RegCliCommand,
+    }
+
     #[test]
     /// Enumerates multiple options of the `pull` command to ensure API doesn't
     /// change between versions. This test will fail if `wash reg pull`
@@ -489,10 +471,11 @@ mod tests {
         // Not explicitly used, just a placeholder for a directory
         const TESTDIR: &str = "./tests/fixtures";
 
-        let pull_basic = RegCli::from_iter(&["reg", "pull", ECHO_WASM]);
-        let pull_all_flags =
-            RegCli::from_iter(&["reg", "pull", ECHO_WASM, "--allow-latest", "--insecure"]);
-        let pull_all_options = RegCli::from_iter(&[
+        let pull_basic: Cmd = Parser::try_parse_from(&["reg", "pull", ECHO_WASM]).unwrap();
+        let pull_all_flags: Cmd =
+            Parser::try_parse_from(&["reg", "pull", ECHO_WASM, "--allow-latest", "--insecure"])
+                .unwrap();
+        let pull_all_options: Cmd = Parser::try_parse_from(&[
             "reg",
             "pull",
             ECHO_WASM,
@@ -500,21 +483,20 @@ mod tests {
             TESTDIR,
             "--digest",
             "sha256:a17a163afa8447622055deb049587641a9e23243a6cc4411eb33bd4267214cf3",
-            "--output",
-            "text",
             "--password",
             "password",
             "--user",
             "user",
-        ]);
-        match pull_basic.command {
+        ])
+        .unwrap();
+        match pull_basic.reg {
             RegCliCommand::Pull(PullCommand { url, .. }) => {
                 assert_eq!(url, ECHO_WASM);
             }
             _ => panic!("`reg pull` constructed incorrect command"),
         };
 
-        match pull_all_flags.command {
+        match pull_all_flags.reg {
             RegCliCommand::Pull(PullCommand {
                 url,
                 allow_latest,
@@ -528,12 +510,11 @@ mod tests {
             _ => panic!("`reg pull` constructed incorrect command"),
         };
 
-        match pull_all_options.command {
+        match pull_all_options.reg {
             RegCliCommand::Pull(PullCommand {
                 url,
                 destination,
                 digest,
-                output,
                 opts,
                 ..
             }) => {
@@ -543,7 +524,6 @@ mod tests {
                     digest.unwrap(),
                     "sha256:a17a163afa8447622055deb049587641a9e23243a6cc4411eb33bd4267214cf3"
                 );
-                assert_eq!(output.kind, OutputKind::Text);
                 assert_eq!(opts.user.unwrap(), "user");
                 assert_eq!(opts.password.unwrap(), "password");
             }
@@ -561,14 +541,15 @@ mod tests {
 
         // Push echo.wasm and pull from local registry
         let echo_push_basic = &format!("{}/echo:pushbasic", LOCAL_REGISTRY);
-        let push_basic = RegCli::from_iter(&[
+        let push_basic: Cmd = Parser::try_parse_from(&[
             "reg",
             "push",
             echo_push_basic,
             &format!("{}/echopush.wasm", TESTDIR),
             "--insecure",
-        ]);
-        match push_basic.command {
+        ])
+        .unwrap();
+        match push_basic.reg {
             RegCliCommand::Push(PushCommand {
                 url,
                 artifact,
@@ -584,15 +565,16 @@ mod tests {
 
         // Push logging.par.gz and pull from local registry
         let logging_push_all_flags = &format!("{}/logging:allflags", LOCAL_REGISTRY);
-        let push_all_flags = RegCli::from_iter(&[
+        let push_all_flags: Cmd = Parser::try_parse_from(&[
             "reg",
             "push",
             logging_push_all_flags,
             &format!("{}/logging.par.gz", TESTDIR),
             "--insecure",
             "--allow-latest",
-        ]);
-        match push_all_flags.command {
+        ])
+        .unwrap();
+        match push_all_flags.reg {
             RegCliCommand::Push(PushCommand {
                 url,
                 artifact,
@@ -610,7 +592,7 @@ mod tests {
 
         // Push logging.par.gz to different tag and pull to confirm successful push
         let logging_push_all_options = &format!("{}/logging:alloptions", LOCAL_REGISTRY);
-        let push_all_options = RegCli::from_iter(&[
+        let push_all_options: Cmd = Parser::try_parse_from(&[
             "reg",
             "push",
             logging_push_all_options,
@@ -619,21 +601,19 @@ mod tests {
             "--insecure",
             "--config",
             &format!("{}/config.json", TESTDIR),
-            "--output",
-            "json",
             "--password",
             "supers3cr3t",
             "--user",
             "localuser",
-        ]);
-        match push_all_options.command {
+        ])
+        .unwrap();
+        match push_all_options.reg {
             RegCliCommand::Push(PushCommand {
                 url,
                 artifact,
                 opts,
                 allow_latest,
                 config,
-                output,
                 ..
             }) => {
                 assert_eq!(&url, logging_push_all_options);
@@ -643,7 +623,6 @@ mod tests {
                 assert_eq!(config.unwrap(), format!("{}/config.json", TESTDIR));
                 assert_eq!(opts.user.unwrap(), "localuser");
                 assert_eq!(opts.password.unwrap(), "supers3cr3t");
-                assert_eq!(output.kind, OutputKind::Json);
             }
             _ => panic!("`reg push` constructed incorrect command"),
         };
